import React, { useEffect, useMemo, useState } from 'react'
import { CloseOutlined, DeleteOutlined, PlusOutlined } from '@ant-design/icons';
import { Drawer, Form, Input, Card, Space, Button, Upload, Popconfirm, Modal, FormListOperation, FormListFieldData, Tooltip } from 'antd'
import type { RcFile, UploadProps } from 'antd/es/upload';
import type { UploadFile } from 'antd/es/upload/interface';
import './editor.css';

const layout = {
  labelCol: { span: 5, },
  wrapperCol: { span: 15 },
};

interface AttributeValue {
  id?: number;
  attrId?: number;
  valName: string;
  imgId?: number;
  imgUrl?: string;
}

export interface SampleAttribute {
  id: number;
  prototypeId: number;
  attrName: string;
  isContainImg: number;
  attrVals: AttributeValue[]
}

interface CreateSampleAttrProps {
  visible: boolean;
  onCancel: (flag?: boolean) => void;
  onSave: () => void;
  editData?: SampleAttribute[] | null;
}

const getBase64 = (file: RcFile): Promise<string> =>
  new Promise((resolve, reject) => {
    const reader = new FileReader();
    reader.readAsDataURL(file);
    reader.onload = () => resolve(reader.result as string);
    reader.onerror = (error) => reject(error);
  });


const SampleAttrEditor: React.FC<CreateSampleAttrProps> = (props) => {

  const { visible, onCancel, onSave, editData } = props;
  const [saveLoading, setSaveLoading] = useState(false);
  const [previewOpen, setPreviewOpen] = useState(false);
  const [previewImage, setPreviewImage] = useState('');
  const [previewTitle, setPreviewTitle] = useState('');
  const [form] = Form.useForm();

  useEffect(() => {
    if (visible) {
      setInitValue();
    } else {
      form.resetFields();
    }
  }, [visible]);

  async function setInitValue() {
    if (editData) {
      form.setFieldsValue({ attrData: editData });
    }
  }

  const save = async (values: any) => {
    setSaveLoading(true);
    debugger

    setSaveLoading(false);
  }

  const handleCancel = () => setPreviewOpen(false);

  const handlePreview = async (file: UploadFile) => {
    if (!file.url && !file.preview) {
      file.preview = await getBase64(file.originFileObj as RcFile);
    }

    setPreviewImage(file.url || (file.preview as string));
    setPreviewOpen(true);
    setPreviewTitle(file.name || file.url!.substring(file.url!.lastIndexOf('/') + 1));
  };

  const uploadButton = (
    <div className='flex'>
      <PlusOutlined />
      <div style={{ marginLeft: 8 }}>上传</div>
    </div>
  );
  const handleChange: UploadProps['onChange'] = ({ fileList: newFileList }) => {
    // setFileList(newFileList);
  }

  const readerAttrValueEditForm = (subFields: FormListFieldData[], attrVals: AttributeValue[], { remove, add }: FormListOperation, isContainImg: boolean) => {
    return (
      <div style={{ display: 'flex', flexDirection: 'column', rowGap: 16 }}>
        {subFields.map((subField) => {
          const item = attrVals[subField.key];
          const uploadFile: UploadFile | null = item && item.imgUrl && item.imgId ? {
            uid: `${item.imgId}`,
            name: '',
            status: 'done',
            url: item.imgUrl
          } : null
          const fileList: UploadFile[] = [];
          if (uploadFile) {
            fileList.push(uploadFile)
          }
          return (
            <Form.Item {...subField} style={{marginBottom: 0}}>
              <Space key={subField.key}>
                <Form.Item noStyle name={[subField.name, 'valName']}>
                  <Input size='middle' />
                </Form.Item>
                {isContainImg && <Form.Item noStyle shouldUpdate name={[subField.name, 'imgUrl']}>
                  <Upload className='attr-image'
                    action="https://run.mocky.io/v3/435e224c-44fb-4773-9faf-380c5e6a2188"
                    listType="picture-card"
                    onPreview={handlePreview}
                    onChange={handleChange}
                    maxCount={1}
                    defaultFileList={[...fileList]}
                  >
                    {fileList.length > 0 ? null : uploadButton}
                  </Upload>
                </Form.Item>
                }
                <Popconfirm
                  title="删除属性值"
                  description="确认是否删除当前属性值?"
                  onConfirm={() => { remove(subField.name); }}
                  okText="确认"
                  cancelText="取消"
                >
                  <Tooltip title="删除属性值">
                    <DeleteOutlined className='delete-icon'/>
                  </Tooltip>
                </Popconfirm>
              </Space>
            </Form.Item>
          )
        })}
        <Button type="dashed" size='middle' onClick={() => add({ valName: '', imgUrl: '', imgId: 0 })} block>
          + 添加自定义值
        </Button>
      </div>)
  }

  const renderAttrEditForm = (fields: FormListFieldData[], { remove, add }: FormListOperation, dataSource: SampleAttribute[]) => {
    return (
      <div style={{ display: 'flex', rowGap: 16, flexDirection: 'column' }}>
        {fields.map((field) => {
          const data = dataSource[field.key];
          return (
            <Form.Item {...field} style={{marginBottom: 4}}>
              <Card size="small" title={field.name} key={field.key}
                style={{ width: 500 }}
                extra={
                  <Popconfirm
                    title="删除属性"
                    description="确认是否删除当前属性?"
                    onConfirm={() => { remove(field.name); }}
                    okText="确认"
                    cancelText="取消"
                  >
                    <Tooltip title="删除属性">
                      <CloseOutlined style={{ fontSize: '16px' }} className='delete-icon' />
                    </Tooltip>
                  </Popconfirm>
                }
              >
                <Form.Item label="属性名称" name={[field.name, 'attrName']} labelCol={{ span: 4 }}>
                  <Input size='middle' />
                </Form.Item>

                <Form.Item label="属性值" labelCol={{ span: 4 }}>
                  <Form.List name={[field.name, 'attrVals']}>
                    {(subFields, subOpt) => {
                      return readerAttrValueEditForm(subFields, data.attrVals, subOpt, data.isContainImg === 1)
                    }}
                  </Form.List>
                </Form.Item>
              </Card>
            </Form.Item>

          )
        })}

        <Button type="dashed" size='middle' onClick={() => add({
          "prototypeId": 88,
          "attrName": "",
          "isContainImg": 1,
          "attrVals": [
          ]
        },)} block>
          + 添加属性
        </Button>
      </div>
    )
  }

  return (
    <>
      <Drawer
        open={visible}
        title={isEdit ? "编辑" : "新建"}
        width={640}
        onClose={() => { onCancel() }}
        extra={
          <Space>
            <Button type="primary" size='middle' onClick={() => { save }}>
              保存
            </Button>
          </Space>
        }
        destroyOnClose
      >
        <Form
          form={form}
          {...layout}
          onFinish={save}
          labelCol={{ flex: '0 0 100px' }}
          wrapperCol={{ span: 16 }}
        >
          <Form.List name="attrData">
            {(fields, operation) => renderAttrEditForm(fields, operation, form.getFieldsValue()['attrData'])}
          </Form.List>

        </Form>
      </Drawer>
      <Modal open={previewOpen} title={previewTitle} footer={null} onCancel={handleCancel}>
        <img alt="example" style={{ width: '100%' }} src={previewImage} />
      </Modal>
    </>
  )
}

export default SampleAttrEditor;
