import React, { useState, useEffect, useRef } from 'react';
import { Table, Tag, Card, Descriptions } from 'antd';
import type { TableColumnsType, DescriptionsProps } from 'antd';
import { Pie, Liquid, Gauge } from '@ant-design/plots';
import { t } from '@/utils/i18n';
import { useRequest } from '@/hooks/use-request';
import redisService from '@/request/service/redis';
import type { RedisKeyInfo, RedisMonitorInfoVO } from '@/models'

export default () => {

    const [dataSource, setDataSource] = useState<RedisKeyInfo[]>([]);
    const [redisMonitorInfo, setRedisMonitorInfo] = useState<RedisMonitorInfoVO>();
    const graphRef = useRef(null);

    const { runAsync: getPageApi } = useRequest(redisService.getKeyDefineListApi, { manual: true });
    const { runAsync: getCache } = useRequest(redisService.getCacheApi, { manual: true });

    const load = async () => {
        const [error, { data }] = await getPageApi();
        if (!error) {
            setDataSource(data);
        }
    };

    const loadCache = async () => {
        const [error, { data }] = await getCache();
        if (!error) {
            setRedisMonitorInfo(data);
            graphRef.current!!.changeData(parseFloat(data.info.used_memory_human.replace("M", ''))/100)
        }
    }

    const columns: TableColumnsType<RedisKeyInfo> = [
        {
            title: 'Key模板',
            dataIndex: 'keyTemplate',
            key: 'keyTemplate',
            align: 'right',
            width: 200,
        },
        {
            title: 'Key类型',
            dataIndex: 'keyType',
            key: 'keyType',
            align: 'center',
            width: 150
        },
        {
            title: 'Value类型',
            key: 'valueType',
            dataIndex: 'valueType',
            align: 'left',
            width: 200
        },
        {
            title: '超时时间',
            key: 'timeoutType',
            dataIndex: 'timeoutType',
            width: 120,
            align: 'center',
            render: (value: number) => {
                return (value === 3 ? <Tag color="purple">固定超时</Tag> : <Tag color="blue">动态超时</Tag>)
            }
        },
        {
            title: '备注',
            key: 'memo',
            dataIndex: 'memo',
            align: 'left',
        }
    ];

    const items: DescriptionsProps['items'] = [
        {
            key: 'redis_version',
            label: 'Redis版本',
            children: redisMonitorInfo?.info?.redis_version,
        },
        {
            key: 'redis_mode',
            label: '运行模式',
            children: redisMonitorInfo?.info?.redis_mode == 'standalone' ? '单机' : '集群',
        },
        {
            key: 'tcp_port',
            label: '端口',
            children: redisMonitorInfo?.info?.tcp_port,
        },
        {
            key: 'connected_clients',
            label: '客户端数',
            children: redisMonitorInfo?.info?.connected_clients,
        },

        {
            key: 'uptime_in_days',
            label: '运行时间(天)',
            children: redisMonitorInfo?.info?.uptime_in_days,
        },
        {
            key: 'used_memory_human',
            label: '使用内存',
            children: redisMonitorInfo?.info?.used_memory_human,
        },
        {
            key: 'used_cpu_user_children',
            label: '使用CPU',
            children: redisMonitorInfo?.info ? parseFloat(redisMonitorInfo?.info?.used_cpu_user_children).toFixed(2) : '',
        },
        {
            key: 'maxmemory_human',
            label: '内存配置',
            children: redisMonitorInfo?.info?.maxmemory_human,
        },
        {
            key: 'aof_enabled',
            label: 'AOF是否开启',
            children: redisMonitorInfo?.info?.aof_enabled == '0' ? '否' : '是',
        },
        {
            key: 'rdb_last_bgsave_status',
            label: 'RDB是否成功',
            children: redisMonitorInfo?.info?.rdb_last_bgsave_status,
        },
        {
            key: 'dbSize',
            label: 'Key数量',
            children: redisMonitorInfo?.dbSize,
        },
        {
            key: 'instantaneous',
            label: '网络入口/出口',
            children: (<>
                {redisMonitorInfo?.info?.instantaneous_input_kbps} kps/{redisMonitorInfo?.info?.instantaneous_output_kbps} kps
            </>),
        },
    ];

    const config = {
        appendPadding: 10,
        data: redisMonitorInfo?.commandStats.map(it => {
            return {
                type: it.command,
                value: it.usec
            }
        }) ?? [],
        angleField: 'value',
        colorField: 'type',
        radius: 0.8,
        label: {
            type: 'outer',
            content: '{name} {percentage}',
        },
        interactions: [
            {
                type: 'pie-legend-active',
            },
            {
                type: 'element-active',
            },
        ],
    };

    const ticks = [0, 1 / 3, 2 / 3, 1];
    const color = ['#30BF78', '#FAAD14', '#F4664A'];
    const memUsage = {
        percent: 0,
        type: 'meter',
        innerRadius: 0.8,
        range: { ticks, color },
        indicator: {
            pointer: {
                style: {
                    stroke: '#D0D0D0',
                },
            },
            pin: {
                style: {
                    stroke: '#D0D0D0',
                },
            },
        },
        statistic: {
            title: {
                formatter: () => redisMonitorInfo?.info.used_memory_human??'0M',
                style: ({ percent }) => {
                  return {
                    fontSize: '24px',
                    lineHeight: 1,
                    color: percent < ticks[1] ? color[0] : percent < ticks[2] ? color[1] : color[2],
                  };
                },
              },
            content: {
                offsetY: 24,
                style: {
                    fontSize: '16px',
                    lineHeight: '16px',
                },
                formatter: () => '内存消耗'
            },
        },
        onReady: (plot) => {
            graphRef.current = plot;
        }
    }

    useEffect(() => {
        load();
        loadCache();
    }, []);

    return (
        <>
            <div>
                <Card className='mt-[4px] dark:bg-[rgb(33,41,70)] bg-white roundle-lg px[12px]'>
                    <Descriptions title="基本信息"
                        bordered
                        column={6}
                        items={items} />
                </Card>

                <div className='flex flex-row'>
                    <Card className='basis-1/2 mt-[4px] mr-[8px] dark:bg-[rgb(33,41,70)] bg-white roundle-lg px[12px]'
                        title="命令统计">
                        <Pie {...config} />
                    </Card>

                    <Card className='basis-1/2 mt-[4px] ml-[8px] dark:bg-[rgb(33,41,70)] bg-white roundle-lg px[12px]'
                        title="内存使用情况">
                        <Gauge {...memUsage} />
                    </Card>
                </div>

                <Card className='mt-[4px] dark:bg-[rgb(33,41,70)] bg-white roundle-lg px[12px]'>
                    <Table rowKey="id"
                        scroll={{ x: true }}
                        columns={columns}
                        dataSource={dataSource}
                        className='bg-transparent'
                        pagination={false} />
                </Card>
            </div>
        </>
    );
};

