import React, { useEffect, useState } from 'react';
import { Form, Input, InputNumber, Radio, Modal, Steps, DatePicker, Button } from 'antd';
import platformShopService from '@/request/service/platform-shop';
import { useRequest } from '@/hooks/use-request';
import type { PlatformShop } from '@/models'
import { antdUtils } from '@/utils/antd';
import customParseFormat from 'dayjs/plugin/customParseFormat';

import dayjs from 'dayjs';
dayjs.extend(customParseFormat);


const layout = {
    labelCol: { span: 4, },
    wrapperCol: { span: 16 }
};

interface EditorProps {
    visible: boolean;
    onCancel: (flag?: boolean) => void;
    onSave: (role: PlatformShop) => void;
    data?: PlatformShop | null;
}

const TenantEditor: React.FC<EditorProps> = (props) => {

    const { visible, onCancel, onSave, data } = props;

    const { runAsync: updateApi } = useRequest(platformShopService.updatePlatformShop, { manual: true });
    const { runAsync: createApi } = useRequest(platformShopService.createPlatformShop, { manual: true });

    const isEdit = !!data;

    const [current, setCurrent] = useState(0);

    const next = async () => {
        const isValidate = await form.validateFields();
        console.log(isValidate)
        setCurrent(current + 1);
    };

    const prev = () => {
        setCurrent(current - 1);
    };
    const [saveLoading, setSaveLoading] = useState(false);
    const [form] = Form.useForm();
    const [shopForm] = Form.useForm();

    useEffect(() => {
        if (visible) {
            if (data) {
                form.setFieldsValue(data);
                form.setFieldValue('expiresTime', dayjs(data.expiresTime));
                shopForm.setFieldsValue(data.shopConfig);
            } else {
                form.setFieldValue("shopStatus", 0);
            }
        } else {
            form.resetFields();
            shopForm.resetFields();
        }
        setCurrent(0);
    }, [visible]);

    const save = async () => {
        await shopForm.validateFields();
        setSaveLoading(true);
        const fieldValues = form.getFieldsValue();
        const expiresTime = dayjs(form.getFieldValue('expiresTime')).toDate().getTime();
        const shopFieldValues = shopForm.getFieldsValue();
        const newValue = isEdit ? { ...data, ...fieldValues, expiresTime, shopConfig: { ...shopFieldValues } } : { ...fieldValues, expiresTime, shopConfig: { ...shopFieldValues } };
        const [error, { msg, code }] = isEdit ? await updateApi(newValue) : await createApi(newValue);
        if (!error && code === 0) {
            onSave(newValue);
        } else {
            antdUtils.message?.open({
                type: 'error',
                content: msg ?? '操作失败',
            });
        }
        setSaveLoading(false);
    }

    return (
        <>
            <Modal
                open={visible}
                title={isEdit ? "编辑" : "新建"}
                width={640}
                onOk={save}
                onCancel={() => onCancel()}
                footer={null}
                confirmLoading={saveLoading}
                destroyOnClose
            >
                <Steps className="my-5" current={current} items={[
                    {
                        title: '商户设置'
                    },
                    {
                        title: '店铺配置',
                    }
                ]} />

                <Form
                    form={form}
                    {...layout}
                    onFinish={save}
                    labelCol={{ flex: '0 0 100px' }}
                    wrapperCol={{ span: 16 }}
                    hidden={current === 1}

                >
                    <Form.Item name="shopName" label="店铺名称"
                        rules={[
                            {
                                required: true,
                                message: '请输入店铺名称',
                            },
                        ]}
                    >
                        <Input />
                    </Form.Item>

                    <Form.Item name="platformId" label="平台"
                        rules={[
                            {
                                required: true,
                                message: '请选择',
                            },
                        ]}
                    >
                        <Input />
                    </Form.Item>

                    <Form.Item name="userName" label="账号">
                        <Input />
                    </Form.Item>

                    <Form.Item name="password" label="密码">
                        <Input />
                    </Form.Item>

                    <Form.Item name="expiresTime" label="过期时间"
                        rules={[
                            {
                                required: true,
                                message: '请设置过期时间',
                            },
                        ]}>
                        <DatePicker showTime format="YYYY-MM-DD HH:mm:ss" />
                    </Form.Item>

                    <Form.Item name="shopStatus" label="状态" rules={[{
                        required: true,
                        message: '请设置状态',
                    },]}>
                        <Radio.Group options={[
                            { value: 1, label: "开启" },
                            { value: 2, label: "停用" }
                        ]} optionType="default">
                        </Radio.Group>
                    </Form.Item>

                    <Form.Item name="tokenFlag" label="授权状态" rules={[{
                        required: true,
                        message: '请设置授权状态',
                    },]}>
                        <Radio.Group options={[
                            { value: 0, label: "未授权不能为空" },
                            { value: 1, label: "成功" },
                            { value: 2, label: "失败" }
                        ]} optionType="default">
                        </Radio.Group>
                    </Form.Item>
                </Form>

                <Form
                    form={shopForm}
                    {...layout}
                    onFinish={save}
                    labelCol={{ flex: '0 0 100px' }}
                    wrapperCol={{ span: 16 }}
                    hidden={current === 0}
                >
                    <Form.Item name="shopId" label="店铺"
                        rules={[
                            {
                                required: true,
                                message: '请设置店铺',
                            },
                        ]}>
                        <InputNumber />
                    </Form.Item>
                    <Form.Item name="iossNo" label="欧盟税号">
                        <Input />
                    </Form.Item>
                    <Form.Item name="vat" label="vat税号">
                        <Input />
                    </Form.Item>
                    <Form.Item name="language" label="语言">
                        <Input />
                    </Form.Item>
                    <Form.Item name="dayMaxPublishNumber" label="最大刊登量">
                        <InputNumber min={0} />
                    </Form.Item>
                </Form>

                <div className='flex justify-end'>
                    {
                        (current === 0 ?
                            (<Button type='primary' size='large' onClick={next}> 下一步 </Button>) :
                            (
                                <>
                                    <Button size='large' onClick={prev} disabled={saveLoading}> 上一步 </Button>
                                    <Button type='primary' size='large' className='ml-2' onClick={save}> 提交 </Button>
                                </>
                            )
                        )
                    }
                </div>
            </Modal>
        </>
    )
}

export default TenantEditor;
