import {
  ProFormRadio,
  ProFormSwitch,
  ProList,
} from '@ant-design/pro-components';
import { Card, Tag, Form, Input, Space, Select, Button, Popover } from 'antd';
import { useState, useEffect } from 'react';
import { t } from '@/utils/i18n';
import {
  EditOutlined,
  EllipsisOutlined,
  SearchOutlined,
  UndoOutlined,
  PlusOutlined,
  ExclamationCircleFilled,
  DeleteOutlined
} from '@ant-design/icons';
import { antdUtils } from '@/utils/antd';
import { useRequest } from '@/hooks/use-request';
import { formatDate } from '@/utils/formatTime'
import userService from '@/request/service/user';
import { UserVO } from '@/models';
import UserEditor from './create-user';

const renderUserCard = (user: UserVO) => {
  return (
    <div style={{ flex: 1, }}>
      <div>
        <div className=' text-gray-900 text-lg font-semibold leading-relaxed'>{user.username}</div>
        <div className='text-zinc-400 text-xs font-normal leading-relaxed'>{user.remark}</div>
      </div>
      <div className='grid grid-cols-2 gap-4 mt-5' >
        <div>
          <div className='text-zinc-400 text-xs font-normal leading-relaxed'>用户账号</div>
          <div className='text-gray-900 text-xs font-medium leading-relaxed'>{user.username}</div>
        </div>

        <div>
          <div className='text-zinc-400 text-xs font-normal leading-relaxed'>手机号码</div>
          <div className='text-gray-900 text-xs font-medium leading-relaxed'>{user.mobile}</div>
        </div>

        <div>
          <div className='text-zinc-400 text-xs font-normal leading-relaxed'>用户邮箱</div>
          <div className='text-gray-900 text-xs font-medium leading-relaxed'>{user.email}</div>
        </div>

        <div>
          <div className='text-zinc-400 text-xs font-normal leading-relaxed'>最后登录时间</div>
          <div className='text-gray-900 text-xs font-medium leading-relaxed'>{formatDate(new Date(user.loginDate), "YYYY-mm-dd HH:MM:SS")}</div>
        </div>
      </div>
    </div>
  )
}

export default () => {

  const [editorVisable, seEditorVisable] = useState<boolean>(false);
  const [editData, seEditData] = useState<UserVO>();
  const [cardActionProps, setCardActionProps] = useState<'actions' | 'extra'>('actions');

  const [dataSource, setDataSource] = useState<any[]>([]);
  const [searchFrom] = Form.useForm();
  const { runAsync: getUserList } = useRequest(userService.getUserPageApi, { manual: true });
  const { runAsync: deleteUser } = useRequest(userService.deleteUserApi, { manual: true });
  const [open, setOpen] = useState(false);

  const handleOpenChange = (isOpen: boolean) => {
    setOpen(isOpen);
  }

  const showDeleteConfirm = (role: UserVO) => {
    antdUtils.modal?.confirm({
      title: '确认要将该角色删除吗？',
      icon: <ExclamationCircleFilled />,
      content: '请注意删除以后不可恢复！',
      okText: '删除',
      okType: 'danger',
      cancelText: '取消',
      onOk() {
        return new Promise(async (resolve) => {
          const [error, { code, msg }] = await deleteUser(role.id);
          if (error || code !== 0) {
            antdUtils.message?.open({ type: 'error', content: msg ?? '操作失败' })
          } else {
            antdUtils.message?.open({ type: 'success', content: '删除成功' })
          }
          await load();
          resolve('')
        }).catch(() => antdUtils.message?.open({
          type: 'error',
          content: '操作失败',
        }));
      }
    });
  };

  const load = async () => {
    const [error, { code, msg, data }] = await getUserList(searchFrom.getFieldsValue());
    if (!error) {
      setDataSource(data.list.map((item) => {
        return ({
          subTitle: <Tag color="#5BD8A6">{item.dept.name}</Tag>,
          actions: [
            <EditOutlined className='my-1' onClick={() => {
              seEditData(item);
              seEditorVisable(true);
              setOpen(false);
            }} />,
            <DeleteOutlined key='delete' onClick={() => { showDeleteConfirm(item) }} />,
            // <a className='my-1' onClick={() => { showDeleteConfirm(item) }}>删除</a>,
            // <EllipsisOutlined key="ellipsis" />,
            // <a className='my-1'>重置密码</a>,
            // <a className='my-1'>分配角色</a>,
            <div>
            <Popover
              placement="bottom"
              key="ellipsis"
              // open={open}
              // onOpenChange={handleOpenChange}
              content={(
                <div className='flex flex-col'>
                  {/* <a className='my-1' onClick={() => {
                    seEditData(item);
                    seEditorVisable(true);
                    setOpen(false);
                  }}>编辑</a>
                  <a className='my-1' onClick={() => { showDeleteConfirm(item) }}>删除</a> */}
                  <a className='my-1'>重置密码</a>
                  <a className='my-1'>分配角色</a>
                </div>
              )} trigger="hover">
              <a key="actions">
                <EllipsisOutlined />
              </a>
            </Popover>
            </div>,

          ],
          avatar: item.avatar,
          content: renderUserCard(item),
        })
      }))
    }
  }

  useEffect(() => {
    load();
  }, []);


  const onReset = () => {
    searchFrom.resetFields()
    load()
  }

  return (
    <>
      <div>
        <Card className='mt-[4px] dark:bg-[rgb(33,41,70)] bg-white roundle-lg px[12px]' bodyStyle={{ paddingTop: 4, paddingBottom: 4 }}>
          <div className='flex justify-between content-center'>
            <div className='flex justify-normal items-center'>
              <Form layout='inline' form={searchFrom}>
                <Form.Item name="username" label="用户账号">
                  <Input className='w-[150px]' placeholder='请输入账号' allowClear />
                </Form.Item>
                <Form.Item name="mobile" label="手机号码">
                  <Input className='w-[150px]' placeholder='请输入号码' allowClear />
                </Form.Item>
                <Form.Item className='ml-2 w-[130px]' name="status" label="状态">
                  <Select placeholder="请选择" allowClear >
                    <Select.Option value="">全部</Select.Option>
                    <Select.Option value="0">开启</Select.Option>
                    <Select.Option value="1">关闭</Select.Option>
                  </Select>
                </Form.Item>
              </Form>
              <Space.Compact className="ml-5">
                <Button type='primary' size='large' icon={<SearchOutlined />} onClick={load}> 搜索 </Button>
                <Button type='primary' size='large' icon={<UndoOutlined />} onClick={onReset}> 重置 </Button>
              </Space.Compact>
            </div>
            <div className="py-[4px]">
              <Button className="ml-5" type='primary' size='large' icon={<PlusOutlined />} onClick={() => {
                seEditData(undefined);
                seEditorVisable(true);
              }}> 添加用户 </Button>
            </div>
          </div>
        </Card>
        <Card className='mt-[4px] dark:bg-[rgb(33,41,70)] bg-white roundle-lg px[12px]'>
          <div style={{ margin: -24, padding: 24, }}>
            <ProList<any>
              className="pt-2"
              pagination={{
                defaultPageSize: 12,
                showSizeChanger: false,
              }}
              showActions="hover"
              rowClassName="min-w-[360px]"
              rowSelection={false}
              grid={{ gutter: 16, column: 4 }}
              onItem={(record: any) => {
                return {
                  onMouseEnter: () => {
                    console.log(record);
                  },
                  onClick: () => {
                    console.log(record);
                  },
                };
              }}
              metas={{
                title: {},
                subTitle: {},
                type: {},
                avatar: {},
                content: {},
                actions: {
                  cardActionProps,
                },
              }}
              dataSource={dataSource}
            />
          </div>
        </Card>
      </div>
      <UserEditor
        onSave={() => {
          load();
          seEditorVisable(false);
        }}
        onCancel={() => { seEditorVisable(false) }}
        visible={editorVisable}
        data={editData} />
    </>
  );
};
